/* Mdulo NSL que construye los esquemas perceptuales de los landmarks incluidos
   en la vista actual de la rata. Tales esquemas codifican la distancia y el
   ngulo hacia el landmark en cuestin.
   Alejandra Barrera
   Fecha: 14 de agosto de 2006
   Fecha: 19 de febrero de 2007
*/

import nslj.src.system.*; 
import nslj.src.cmd.*; 
import nslj.src.lang.*; 
import nslj.src.math.*; 
import nslj.src.display.*; 
import java.lang.*;

public class LandmarksPerceptualSchema extends NslModule
{
	public NslDinDouble1 distLandmarks;
	public NslDinInt1 angleLandmarks;
	public NslDoutDouble1 psLand1;
	public NslDoutDouble1 psLand2;
	public NslDoutDouble1 psLand3;
        public NslDoutDouble1 psLand4;
	private int sizePS;
	private double ps1Aux[];
	private double ps2Aux[];
	private double ps3Aux[];
      	private double ps4Aux[];
	private boolean inicio;
	private double distLandmarksAux[];
	private int angleLandmarksAux[];
        private double dMaxCyan= 11D;
        private double dMaxBlue= 12D;
        private double dMaxWhite= 6D;
        private double dMaxYellow= 11D;
        
	public LandmarksPerceptualSchema(String nslName, NslModule nslParent,int t)
	{
		super(nslName, nslParent);
		sizePS= t;
		inicio= true;
		
		distLandmarks= new NslDinDouble1("distLandmarks",this, 4); 
		angleLandmarks= new NslDinInt1("angleLandmarks",this, 4); 
		distLandmarksAux= new double[4];
		angleLandmarksAux= new int[4];

		psLand1= new NslDoutDouble1("psLand1",this,sizePS); 
		psLand2= new NslDoutDouble1("psLand2",this,sizePS);
		psLand3= new NslDoutDouble1("psLand3",this,sizePS);
                psLand4= new NslDoutDouble1("psLand4",this,sizePS);
                ps1Aux= new double[sizePS];
		ps2Aux= new double[sizePS];
		ps3Aux= new double[sizePS];
                ps4Aux= new double[sizePS];
	}

	public void simRun()
	{
		distLandmarksAux= distLandmarks.get();
		angleLandmarksAux= angleLandmarks.get();		
		for (int i=0; i<sizePS; i++)
		{
			ps1Aux[i]= 0.0;
			ps2Aux[i]= 0.0;
			ps3Aux[i]= 0.0;
			ps4Aux[i]= 0.0;                        
		}
                double daux1;
                int daux2;
                double d= 3D;
		if (distLandmarksAux[0]!=-1D && angleLandmarksAux[0]!=-1)
		{
			daux1= distLandmarksAux[0]/dMaxCyan; 
			daux2= (int)((1.0D - daux1) * 76D + 2D);                        
                        switch(angleLandmarksAux[0])
                        {
                            case -90:
                                for(int i = 0; i < 80; i++)
                                    ps1Aux[i]= Math.exp((double)(-1 * (i - 22) * (i - 22)) / (2D * (d * d)));
                                break;
                            case 0:
                                for(int i = 0; i < 80; i++)
                                    ps1Aux[i]= Math.exp((double)(-1 * (i - 40) * (i - 40)) / (2D * (d * d)));
                                break;
                            case 90:
                                for(int i = 0; i < 80; i++)
                                    ps1Aux[i]= Math.exp((double)(-1 * (i - 58) * (i - 58)) / (2D * (d * d)));
                                break;
                            default:
                                break;
                        }
 
			for (int neuron=0; neuron<80; neuron++) 
                            ps1Aux[neuron+80]= Math.exp((double)(-1 * Math.pow(neuron-daux2, 2)) / (2D * (d * d)));
		}
		if (distLandmarksAux[1]!=-1D && angleLandmarksAux[1]!=-1)
		{
			daux1= distLandmarksAux[1]/dMaxBlue; 
			daux2= (int)((1.0D - daux1) * 76D + 2D);
                        switch(angleLandmarksAux[1])
                        {
                            case -90:
                                for(int i = 0; i < 80; i++)
                                    ps2Aux[i]= Math.exp((double)(-1 * (i - 22) * (i - 22)) / (2D * (d * d)));
                                break;
                            case 0:
                                for(int i = 0; i < 80; i++)
                                    ps2Aux[i]= Math.exp((double)(-1 * (i - 40) * (i - 40)) / (2D * (d * d)));
                                break;
                            case 90:
                                for(int i = 0; i < 80; i++)
                                    ps2Aux[i]= Math.exp((double)(-1 * (i - 58) * (i - 58)) / (2D * (d * d)));
                                break;
                            default:
                                break;
                        }
 
			for (int neuron=0; neuron<80; neuron++) 
                            ps2Aux[neuron+80]= Math.exp((double)(-1 * Math.pow(neuron-daux2, 2)) / (2D * (d * d)));
		}
		if (distLandmarksAux[2]!=-1D && angleLandmarksAux[2]!=-1)
		{
			daux1= distLandmarksAux[2]/dMaxWhite; 
			daux2= (int)((1.0D - daux1) * 76D + 2D);
                        switch(angleLandmarksAux[2])
                        {
                            case -90:
                                for(int i = 0; i < 80; i++)
                                    ps3Aux[i]= Math.exp((double)(-1 * (i - 22) * (i - 22)) / (2D * (d * d)));
                                break;
                            case 0:
                                for(int i = 0; i < 80; i++)
                                    ps3Aux[i]= Math.exp((double)(-1 * (i - 40) * (i - 40)) / (2D * (d * d)));
                                break;
                            case 90:
                                for(int i = 0; i < 80; i++)
                                    ps3Aux[i]= Math.exp((double)(-1 * (i - 58) * (i - 58)) / (2D * (d * d)));
                                break;
                            default:
                                break;
                        }
 
			for (int neuron=0; neuron<80; neuron++) 
                            ps3Aux[neuron+80]= Math.exp((double)(-1 * Math.pow(neuron-daux2, 2)) / (2D * (d * d)));
		} 
		if (distLandmarksAux[3]!=-1D && angleLandmarksAux[3]!=-1)
		{
			daux1= distLandmarksAux[3]/dMaxYellow; 
			daux2= (int)((1.0D - daux1) * 76D + 2D);
                        switch(angleLandmarksAux[3])
                        {
                            case -90:
                                for(int i = 0; i < 80; i++)
                                    ps4Aux[i]= Math.exp((double)(-1 * (i - 22) * (i - 22)) / (2D * (d * d)));
                                break;
                            case 0:
                                for(int i = 0; i < 80; i++)
                                    ps4Aux[i]= Math.exp((double)(-1 * (i - 40) * (i - 40)) / (2D * (d * d)));
                                break;
                            case 90:
                                for(int i = 0; i < 80; i++)
                                    ps4Aux[i]= Math.exp((double)(-1 * (i - 58) * (i - 58)) / (2D * (d * d)));
                                break;
                            default:
                                break;
                        }
 
			for (int neuron=0; neuron<80; neuron++) 
                            ps4Aux[neuron+80]= Math.exp((double)(-1 * Math.pow(neuron-daux2, 2)) / (2D * (d * d)));
		} 
		psLand1.set(ps1Aux);
		psLand2.set(ps2Aux);
		psLand3.set(ps3Aux);
		psLand4.set(ps4Aux);                
	} //simRun
}	
