/* Mdulo NSL que implementa el modelo de cognicin espacial.
   Alejandra Barrera
   Versin: 1
   Fecha: 10 de marzo de 2005.
*/


import nslj.src.system.*; 
import nslj.src.cmd.*; 
import nslj.src.lang.*; 
import nslj.src.math.*; 
import nslj.src.display.*; 
import java.lang.*;


public class WGModel extends NslModel
{
        private World world;
	private Rat rat;
	private Drive drive;
	private AffPerceptualSchema ps;
	private MotivationalSchema mot;
	private ActionSelectionSchema actionSel;
	private DynamicRemappingLayer drl;
	private PathIntegrationFeatureDetectorLayer piFDL;
	private LandmarksPerceptualSchema landPS;
	private LandmarksFeatureDetectorLayer landFDL;
	private LandmarksLayer landL;
	private InputToPlaceCellLayer inputPCL;
	private PlaceCellLayer pcl;
	private WorldGraphLayer wgl;

	public WGModel()
	{
		super("WGModel",(NslModel)null);
		if (NslMain.TopLoaded) 
		{ 
			System.err.println("ERROR: construction without (name,parent)");
		 	System.exit(1);
		} 
		NslMain.TopLoaded=true;
                
                world= new World("world",this,80,80,90);
                rat= new Rat("rat",this,80,80);	
		ps= new AffPerceptualSchema("ps",this,80);
                landPS= new LandmarksPerceptualSchema("landPS",this,160);
		landFDL= new LandmarksFeatureDetectorLayer("landFDL",this);
		landL= new LandmarksLayer("landL",this);
		drive= new Drive("drive",this);		
		drl= new DynamicRemappingLayer("drl",this,25,25,11,19);
		piFDL= new PathIntegrationFeatureDetectorLayer("piFDL",this);
		inputPCL= new InputToPlaceCellLayer("inputPCL",this);
		pcl= new PlaceCellLayer("pcl",this);
		wgl= new WorldGraphLayer("wgl",this);
		mot= new MotivationalSchema("mot",this);
		actionSel= new ActionSelectionSchema("actionSel",this);
		
		initSys();
	}

	public WGModel(String nslName, NslModule nslParent) 
	{
		super(nslName, nslParent);
                
                world= new World("world",this,80,80,90);
                rat= new Rat("rat",this,80,80);	
		ps= new AffPerceptualSchema("ps",this,80);
                landPS= new LandmarksPerceptualSchema("landPS",this,160);
		landFDL= new LandmarksFeatureDetectorLayer("landFDL",this);
		landL= new LandmarksLayer("landL",this);
		drive= new Drive("drive",this);		
		drl= new DynamicRemappingLayer("drl",this,25,25,11,19);
		piFDL= new PathIntegrationFeatureDetectorLayer("piFDL",this);
		inputPCL= new InputToPlaceCellLayer("inputPCL",this);
		pcl= new PlaceCellLayer("pcl",this);
		wgl= new WorldGraphLayer("wgl",this);
		mot= new MotivationalSchema("mot",this);
		actionSel= new ActionSelectionSchema("actionSel",this);
		
	   	initSys();
	}
        
    public  void initSys() 
	{
		system.setRunEndTime(10);
		system.nslSetRunDelta(0.1);
                system.setNumRunEpochs(30);
	}

	public void makeConn()
	{
                nslConnect(world.currentHeadAngle, rat.headAngleRat);
                nslConnect(world.colorMatrixO, rat.colorMatrixO);
                nslConnect(world.colorMatrixR, rat.colorMatrixR);
                nslConnect(world.colorMatrixL, rat.colorMatrixL);
                nslConnect(rat.yellowO, ps.yellowO);
                nslConnect(rat.yellowR, ps.yellowR);
                nslConnect(rat.yellowL, ps.yellowL);
                nslConnect(rat.blueO, ps.blueO);
                nslConnect(rat.blueR, ps.blueR);
                nslConnect(rat.blueL, ps.blueL);
                nslConnect(rat.redO, ps.redO);
                nslConnect(rat.redR, ps.redR);
                nslConnect(rat.redL, ps.redL);
                nslConnect(rat.roseO, ps.roseO);
                nslConnect(rat.roseR, ps.roseR);
                nslConnect(rat.roseL, ps.roseL);
                nslConnect(rat.whiteO, ps.whiteO);
                nslConnect(rat.whiteR, ps.whiteR);
                nslConnect(rat.whiteL, ps.whiteL);
                nslConnect(rat.currentHeadAngleRat, ps.headAngleRat);		
		nslConnect(rat.distFood, drive.distFood);
		nslConnect(rat.turnToFood, mot.turnToFood);
		nslConnect(rat.distLandmarks, landPS.distLandmarks);
		nslConnect(rat.angleLandmarks, landPS.angleLandmarks);
		nslConnect(landPS.psLand1, landFDL.psLand1);
		nslConnect(landPS.psLand2, landFDL.psLand2);
		nslConnect(landPS.psLand3, landFDL.psLand3);
                nslConnect(landPS.psLand4, landFDL.psLand4);
		nslConnect(landFDL.fdl1, landL.fdl1);
		nslConnect(landFDL.fdl2, landL.fdl2);
		nslConnect(landFDL.fdl3, landL.fdl3);
                nslConnect(landFDL.fdl4, landL.fdl4);
		nslConnect(landFDL.activation4Land, landL.activation4Land);
		nslConnect(actionSel.newHeadAngleRat, world.newHeadAngleRat);
		nslConnect(actionSel.changeXRat, world.changeXRat);
		nslConnect(actionSel.changeYRat, world.changeYRat);
                nslConnect(actionSel.changeZRat, world.changeZRat);
		nslConnect(ps.affPS, mot.affPS);
		nslConnect(ps.currentHeadAngleRat, actionSel.headAngleRat);
		nslConnect(mot.AngleToGo, actionSel.AngleToGo);
		nslConnect(actionSel.ActionTaken, world.ActionTaken);
		nslConnect(actionSel.currentDir, drl.currentDir);
		nslConnect(actionSel.nextDir, drl.nextDir);
		nslConnect(drl.PIps1dim, piFDL.PIps1dim);
		nslConnect(piFDL.piFDL1dim, inputPCL.piFDL1dim);
		nslConnect(landL.lFDL1dim, inputPCL.lFDL1dim);
		nslConnect(inputPCL.iPCL, pcl.iPCL);
		nslConnect(ps.currentHeadAngleRat, wgl.currentHeadAngleRat);
		nslConnect(pcl.pcl1dim, wgl.pcl1dim);
		nslConnect(drive.reward, wgl.reward);
		nslConnect(rat.distFood, wgl.distFood);
		nslConnect(actionSel.ActionTaken, wgl.ActionTaken);
		nslConnect(ps.currentHeadAngleRat, mot.currentHeadAngleRat);
		nslConnect(wgl.expCycling, mot.expCycling);
		nslConnect(wgl.flagCycling, mot.flagCycling);
	}
}	
