/* Mdulo NSL que representa el mapa del ambiente.

   Alejandra Barrera
   Versin 1
   Fecha: 5 de abril de 2005
*/



import nslj.src.system.*; 
import nslj.src.cmd.*; 
import nslj.src.lang.*; 
import nslj.src.math.*; 
import nslj.src.display.*; 
import java.lang.*;
import java.util.Random;
import java.io.*;
import java.util.*;
import java.awt.*;

public class WorldGraphLayer extends NslModule 
{
	public NslDinInt0 currentHeadAngleRat;
	public NslDinDouble1 pcl1dim;
	public NslDinDouble0 reward;
	public NslDinDouble0 distFood;
	public NslDinInt0 ActionTaken;
	public NslDoutDouble1 expCycling;
	public NslDoutInt1 flagCycling;
	private int x;
	private int y;
	private final int SPIKES = 20;
	private final int DIM1= 20;
	private final int DIM2= 20;   
	private final int TOTAL_NEURONS = DIM1*DIM2;
	private int node_id;
    private int node_id_WG;
    private double number_of_steps;
    private double edge_to_direction[];
    private int previous_direction;
    public static int CurrentWGNode;
    public static int FromWGNode;
    private int CurrentNode;
    private int winner_x;
    private int winner_y;
    private LinkedList pattern_list;
    public static LinkedList map_list;
    private Frame WGFrame;
    private boolean newPatNode;
	private int ActionTakenAux;
	private double dFoodAux;
	private boolean inicio;
	private ActorCriticWG ACWG;
    private final double WG_ALPHA_R = 0.041000000000000002D;
    private final int CYCLING = 3;
    private double expCyclingAux[];
    private boolean flagCyclingAuxB[];
    private int flagCyclingAuxI[];

	public WorldGraphLayer(String nslName, NslModule nslParent)
	{
		super(nslName, nslParent);
		currentHeadAngleRat= new NslDinInt0("currentHeadAngleRat",this);
		pcl1dim= new NslDinDouble1("pcl1dim",this,TOTAL_NEURONS);
		distFood= new NslDinDouble0("distFood",this);
		reward= new NslDinDouble0("reward",this);
		ActionTaken= new NslDinInt0("ActionTaken",this);
		expCycling= new NslDoutDouble1("expCycling",this,8);
		flagCycling= new NslDoutInt1("flagCycling",this,8);
		
		expCyclingAux= new double[8];
		flagCyclingAuxB= new boolean[8];
		flagCyclingAuxI= new int[8];
		dFoodAux= -1D;
		inicio= true;
		ActionTakenAux= 4;
        node_id = 0;
        node_id_WG = 0;
        number_of_steps = 0.0D;
        edge_to_direction = new double[8];
        previous_direction = -1;
        CurrentWGNode = -1;
        FromWGNode = -1;
        CurrentNode = -1;
        winner_x = 0;
        winner_y = 0;
        pattern_list = new LinkedList();
        map_list = new LinkedList();
        for(int i = 0; i < 8; i++)
            edge_to_direction[i] = 0.0D;
        
        ACWG= new ActorCriticWG(TOTAL_NEURONS);
        WGFrame= new WorldGraphFrame();
        WGFrame.setSize(500, 400);
        WGFrame.setVisible(true);
	}

	public void simRun()
	{
		if (!inicio)
		{
			dFoodAux= distFood.get();
			ActionTakenAux= ActionTaken.get();
			if (!Rat.habituation && Rat.newTrial)
				ACWG.initTraces();
		}
		else
			inicio= false;
                double rhatAux= 0.0;
		if (!Rat.habituation)
                {
                    ACWG.initReinforce(ActionTakenAux, 1, pcl1dim.get(), dFoodAux);
                    rhatAux= ACWG.reinforce(reward.get(), 1, pcl1dim.get(), dFoodAux);
                }
		boolean newMapNode= false;
		int currentDirection= 0;
		
		if (currentHeadAngleRat.get()!=0)
			currentDirection= angle2direction(currentHeadAngleRat.get());
		newMapNode= manageMap(currentDirection, Rat.newTrial, pcl1dim.get(), AffPerceptualSchema.differentAff, Drive.ateFood, rhatAux);

		WGFrame.repaint();
		
        for(int j = 0; j < 8; j++)
        {
        	expCyclingAux[j] = 0.0D;
            flagCyclingAuxB[j] = false;
            flagCyclingAuxI[j] = 0;
        }
        if (!Rat.habituation)
        {
            readActivationMap(0, expCyclingAux, flagCyclingAuxB, 1, -999, 0.0D);
            for(int j1 = 0; j1 < 8; j1++)
            {
        	if (flagCyclingAuxB[j1] == true)
            	flagCyclingAuxI[j1] = 1;
            }
        }
                expCycling.set(expCyclingAux);
		flagCycling.set(flagCyclingAuxI);

	}

    public boolean manageMap(int dir, boolean newTrial, 
        double patternPCL[], boolean differentAff, 
        boolean ateFood, double rhatAux)
    {
        boolean flag = false;
        double d1 = 0.60000000000000004D;
        if((CurrentWGNode != -1 || FromWGNode != -1) && !newTrial)
        {
            if (!Rat.habituation)
                initReinforce(dir, 1);
        }
        newPatNode = pattern_activation(dir, newTrial, patternPCL, d1, ateFood);
        flag = map_activation(dir, newTrial, newPatNode, differentAff, ateFood);
        previous_direction = dir;
        if(!newTrial)
        {
            if (!Rat.habituation)
                reinforce(rhatAux, 1);
        }
        else if (newTrial && Rat.reinforcePath)
        {
            reinforce(rhatAux, 1);
            Rat.reinforcePath= false;
        }
        else if (newTrial && !Rat.reinforcePath)
        {
            if (!Rat.habituation)
                reinforce(Math.abs(rhatAux), 1);
        }       
        if(newTrial)
        {
            if (!Rat.habituation)
                initTraces();
        }
        return flag;
    }
    
    public void initTraces()
    {
    	ListIterator mapIterator;
    	NodeMap mapNode= null;
    	for(mapIterator= map_list.listIterator(); mapIterator.hasNext();)
        {
        	mapNode = (NodeMap)mapIterator.next();
            for(int i = 0; i < 4; i++)
            {
            	for(int j = 0; j < 8; j++)
                	mapNode.e[i][j] = 0.0D;
            }
        }
    }

    public void initReinforce(int i, int j)
    {
    	ListIterator mapIterator;
    	NodeMap mapNode= null;
        int k;
        
        if(CurrentWGNode == -1)
            k = FromWGNode;
        else
            k = CurrentWGNode;
    	for(mapIterator= map_list.listIterator(); mapIterator.hasNext();)
        {
        	mapNode = (NodeMap)mapIterator.next();
            if(mapNode.id == k)
            {
            	mapNode.e[j][i] += 0.00099999999999999978D;
                break;
            }
        }
    }
    
    public void reinforce(double rhatAux, int i)
    {
    	ListIterator mapIterator;
    	NodeMap mapNode= null;

    	for(mapIterator= map_list.listIterator(); mapIterator.hasNext();)
        {
        	mapNode = (NodeMap)mapIterator.next();
            for(int j = 0; j < 8; j++)
            {
            	mapNode.w[i][j] += WG_ALPHA_R * rhatAux * mapNode.e[i][j];
                if(mapNode.w[i][j] < 0.0D)
                	mapNode.w[i][j] = 0.0D;
                mapNode.e[i][j] *= 0.90000000000000002D;
            }
        }
    }
 
    public boolean pattern_activation(int currentDir,  
    	boolean newTrial, double ad[], double d, 
        boolean ateFood)
    {
        int i3 = -1; 
        int j3 = -1;
        double d8 = -1D;
        boolean newPatNode = false;
        boolean flag3 = false;
        ListIterator patIterator;
        ListIterator mapIterator;
        NodePattern patNode = null;
        NodeMap mapNode= null;
        int winner;

        patIterator = pattern_list.listIterator();
        if(!patIterator.hasNext())
        {
        	patNode = new NodePattern(node_id, currentDir);
            for(int k1=0; k1<TOTAL_NEURONS; k1++)
            {
            	double d2 = (int)(ad[k1] * (double)SPIKES);
                patNode.place_IDs[k1][0] = (int)d2;
            }
            patIterator.add(patNode);
            CurrentNode = patNode.id;
            node_id++;
            newPatNode = true;
        } 
        else
        {
        	winner = -1;
            while(patIterator.hasNext()) 
            {
            	patNode = (NodePattern)patIterator.next();
                double d7 = 0.0D;
                double d13 = 0.0D;
                for(int k2 = 0; k2 < ad.length; k2++)
                {
                	double d3;
                    if(patNode.place_IDs[k2][0] == 0)
                    	d3 = 0.0D;
                    else
                        d3 = (double)patNode.place_IDs[k2][0] / (double)SPIKES;
                    if(d3 > ad[k2])
                        d7 += ad[k2];
                    else
                        d7 += d3;
                    d13 += ad[k2];
                }
                double d10;
                if(d7 == 0.0D)
                    d10 = 0.0D;
                else
                    d10 = d7 / d13;
                if(d10 > d && d10 > d8)
                {
                    winner = patNode.id;
                    d8 = d10;
                }
            }
            boolean flag4 = false;
            for(patIterator= pattern_list.listIterator(); patIterator.hasNext();)
            {
            	patNode = (NodePattern)patIterator.next();
                if(patNode.id == winner)
                {
                	if(patNode.dir != currentDir && ateFood)
                    	flag4 = true;
                    break;
                }
            }
            if(flag4)
            {
            	winner = -1;
                for(patIterator= pattern_list.listIterator(); patIterator.hasNext();)
                {
	                 patNode = (NodePattern)patIterator.next();
                     if(patNode.dir == currentDir)
                     {
                     	winner = patNode.id;
                        break;
                     }
                }
            }
            if(winner != -1 && currentDir != previous_direction && !newTrial && (patNode.mapNodeID != CurrentWGNode && CurrentWGNode != -1 || patNode.mapNodeID != FromWGNode && FromWGNode != -1))
            {
            	if(FromWGNode != -1)
                {
                	flag3 = true;
                } else
                {
                    for(patIterator= pattern_list.listIterator(); patIterator.hasNext();)
                    {
 	                   patNode= (NodePattern)patIterator.next();
                       if(patNode.id == winner)
                       {
                        	i3 = patNode.mapNodeID;
                            break;
                       }
                    }
                    for(patIterator= pattern_list.listIterator(); patIterator.hasNext();)
                    {
 	                   patNode= (NodePattern)patIterator.next();
                       if(patNode.id == CurrentNode)
                       {
                       		j3 = patNode.mapNodeID;
                            break;
                       }
                    }
			        if(i3 != j3)
			        {
            	        mergeNodes(i3, j3);
            	 	}
                    CurrentWGNode = i3;
                }
            } else
            if(winner != -1 && currentDir == previous_direction && !newTrial && (patNode.mapNodeID != CurrentWGNode && CurrentWGNode != -1 || patNode.mapNodeID != FromWGNode && FromWGNode != -1))
            {
	            for(mapIterator= map_list.listIterator(); mapIterator.hasNext();)
                {
	                mapNode= (NodeMap)mapIterator.next();
                    if(CurrentWGNode == -1 ? mapNode.id == FromWGNode : mapNode.id == CurrentWGNode)
	                    break;
                }
                for(patIterator= pattern_list.listIterator(); patIterator.hasNext();)
                {
	                patNode = (NodePattern)patIterator.next();
                    if(patNode.id == winner)
                    	break;
                }
                for(int l2= 0; l2 < mapNode.pointerWGtoWG.size(); l2++)
                {
	                 Edge edge = (Edge)mapNode.pointerWGtoWG.get(l2);
                     if(edge.points_to_node == patNode.mapNodeID && edge.direction != currentDir)
                     {
	                     winner = -1;
                         l2 = mapNode.pointerWGtoWG.size();
                     }
                }
            }
            if(winner == -1)
            {
            	double d11 = 1.0D;
                patNode = new NodePattern(node_id, currentDir);
                if(currentDir != previous_direction && !newTrial && CurrentWGNode != -1)
                {
	                newPatNode= false;
                    patNode.mapNodeID= CurrentWGNode;
                } else
                {
                    newPatNode= true;
                }
                for(int i2 = 0; i2 < TOTAL_NEURONS; i2++)
                {
                	double d4 = (int)(ad[i2] * (double)SPIKES);
                    patNode.place_IDs[i2][0] = (int)d4;
                }
                patIterator.add(patNode);
                node_id++;
                CurrentNode = patNode.id;
            } else
            {
            	CurrentNode = winner;
                for(patIterator= pattern_list.listIterator(); patIterator.hasNext();)
                {
	                patNode = (NodePattern)patIterator.next();
                    if(patNode.id == CurrentNode)
                    {
                    	for(int j2 = 0; j2 < TOTAL_NEURONS; j2++)
                        {
                        	double d5 = (int)(ad[j2] * (double)SPIKES);
                            patNode.place_IDs[j2][0] = (int)d5;
                        }
                        break;
                    }
                }
                if(!flag3)
	                newPatNode= false;
            }
        }
        return newPatNode;
    }  
    
    public void mergeNodes(int i, int j)
    {
        ListIterator patIterator;
        ListIterator mapIterator;
        NodePattern patNode= null;
        NodeMap mapNode= null;
        NodeMap mapNode2= null;
        for(patIterator = pattern_list.listIterator(); patIterator.hasNext();)
        {
            patNode = (NodePattern)patIterator.next();
            if(patNode.mapNodeID == j)
                patNode.mapNodeID = i;
        }
        for(mapIterator= map_list.listIterator(); mapIterator.hasNext();)
        {
 	        mapNode= (NodeMap)mapIterator.next();
            if(mapNode.id == i)
            	break;
        }
        int j2 = -1;
        boolean flag1 = false;
        for(int k = 0; k < mapNode.pointerWGtoWG.size(); k++)
        {
        	Edge edge = (Edge)mapNode.pointerWGtoWG.get(k);
            if(edge.points_to_node == j)
            {
            	flag1 = true;
                j2 = k;
                k = mapNode.pointerWGtoWG.size();
            }
        }
        if(flag1)
        	mapNode.pointerWGtoWG.remove(j2);
        for(mapIterator= map_list.listIterator(); mapIterator.hasNext();)
        {
	        mapNode2= (NodeMap)mapIterator.next();
            if(mapNode2.id == j)
            	break;
        }
        for(int l = 0; l < mapNode2.pointerWGtoWG.size(); l++)
        {
        	Edge edge1 = (Edge)mapNode2.pointerWGtoWG.get(l);
            if(edge1.points_to_node != i)
            {
            	boolean flag = false;
                for(int i1 = 0; i1 < mapNode.pointerWGtoWG.size(); i1++)
                {
                	Edge edge6 = (Edge)mapNode.pointerWGtoWG.get(i1);
                    if(edge6.points_to_node == edge1.points_to_node)
                    {
                    	flag = true;
                        i1 = mapNode.pointerWGtoWG.size();
                    }
                }
                if(!flag)
                	mapNode.add_edge(edge1.x, edge1.y, edge1.direction, edge1.points_to_node, edge1.steps, edge1.trace);
            }
        }
        boolean flag2 = false;
        for(mapIterator= map_list.listIterator(); mapIterator.hasNext();)
        {
        	NodeMap mapNode3= (NodeMap)mapIterator.next();
            if(mapNode3.id == j)
            {
            	flag2 = true;
                break;
            }
        }
        if(flag2)
        	mapIterator.remove();
        for(mapIterator= map_list.listIterator(); mapIterator.hasNext();)
        {
	        mapNode= (NodeMap)mapIterator.next();
            if(mapNode.id == i)
            	break;
        }
        for(mapIterator= map_list.listIterator(); mapIterator.hasNext();)
        {
        	byte byte1 = -1;
            mapNode2= (NodeMap)mapIterator.next();
            if(mapNode2.id != i)
            {
            	for(int j1 = 0; j1 < mapNode2.pointerWGtoWG.size(); j1++)
                {
                	Edge edge2 = (Edge)mapNode2.pointerWGtoWG.get(j1);
                    if(edge2.points_to_node == j)
                    {
                    	byte1 = 0;
                        j1 = mapNode2.pointerWGtoWG.size();
                    }
                }
                if(byte1 == 0)
                {
                	for(int k1 = 0; k1 < mapNode2.pointerWGtoWG.size(); k1++)
                    {
                    	Edge edge3 = (Edge)mapNode2.pointerWGtoWG.get(k1);
                        if(edge3.points_to_node == i)
                        {
                        	byte1 = 1;
                            k1 = mapNode2.pointerWGtoWG.size();
                        }
                    }
                    if(byte1 == 0)
                    {
                    	for(int l1 = 0; l1 < mapNode2.pointerWGtoWG.size(); l1++)
                        {
                        	Edge edge4 = (Edge)mapNode2.pointerWGtoWG.get(l1);
                            if(edge4.points_to_node == j)
                            {
                            	edge4.points_to_node = mapNode.id;
                                edge4.x = mapNode.wg_x;
                                edge4.y = mapNode.wg_y;
                                l1 = mapNode2.pointerWGtoWG.size();
                            }
                        }
                    } else
                    if(byte1 == 1)
                    {
                    	int k2 = -1;
                        for(int i2 = 0; i2 < mapNode2.pointerWGtoWG.size(); i2++)
                        {
                        	flag1 = false;
                            Edge edge5 = (Edge)mapNode2.pointerWGtoWG.get(i2);
                            if(edge5.points_to_node == j)
                            {
                            	flag1 = true;
                                k2 = i2;
                                i2 = mapNode2.pointerWGtoWG.size();
                            }
                        }
                        if(flag1)
                        	mapNode2.pointerWGtoWG.remove(k2);
                    }
                }
            }
        }
    }
    
    public boolean map_activation(int currentDir, 
        boolean newTrial, boolean newPatNode, 
    	boolean differentAff, boolean ateFood)
    {
        boolean newMapNode = false;
        Direction direction = new Direction(currentDir);
        byte byte0 = -1;
        ListIterator mapIterator;
        ListIterator patIterator;
        NodeMap mapNode= null;
        NodePattern patNode= null;
    
        if (newTrial){
         for(mapIterator = map_list.listIterator(); mapIterator.hasNext();){
             	NodeMap mapNodeAux = (NodeMap)mapIterator.next();
                for(int i = 0; i < mapNodeAux.pointerWGtoWG.size(); i++){
                    Edge edge1 = (Edge)mapNodeAux.pointerWGtoWG.get(i);
                    edge1.trace= false;
                 }
         }
        }
	
        mapIterator= map_list.listIterator();
        if(!mapIterator.hasNext())
        {
            mapNode= new NodeMap(node_id_WG);
            mapIterator.add(mapNode);
            for(patIterator= pattern_list.listIterator(); patIterator.hasNext();)
            {
            	patNode = (NodePattern)patIterator.next();
                if(patNode.id == CurrentNode)
                	break;
            }
            Dimension dimension1 = WGFrame.getSize();
            mapNode.wg_x = dimension1.width / 2;
            mapNode.wg_y = dimension1.height / 2 + dimension1.height / 4;
            patNode.mapNodeID = mapNode.id;
            CurrentWGNode = mapNode.id;
            newMapNode = true;
            for(int i1 = 0; i1 < 8; i1++)
                edge_to_direction[i1] = 0.0D;
            number_of_steps = 0.0D;
        } else
        if(newPatNode)
        {
        	for(patIterator= pattern_list.listIterator(); patIterator.hasNext();)
            {
 		        patNode= (NodePattern)patIterator.next();
                if(patNode.id == CurrentNode)
                	break;
            }
            boolean flag7 = true;
            if(!ateFood)
	        if(!differentAff && !newTrial)
                {
                    NodePattern lastPat= null;
                    NodePattern patAux= null;
                    lastPat= patNode;
                    ListIterator patIteratorAux;
                    for(patIteratorAux= pattern_list.listIterator(); patIteratorAux.hasNext();)
                    {
 		        patAux= (NodePattern)patIteratorAux.next();
                        if((CurrentWGNode != -1 && patAux.mapNodeID == CurrentWGNode && patAux.dir == currentDir) || (FromWGNode != -1 && patAux.mapNodeID == FromWGNode && patAux.dir == currentDir))
                            break;
                    }
                    for(int k=0; k<TOTAL_NEURONS; k++)
                    {
                        double prom = (lastPat.place_IDs[k][0] + patAux.place_IDs[k][0])/2.0;
                        patAux.place_IDs[k][0] = (int)prom;
                    }
                    patIterator.remove();
                    edge_to_direction[currentDir]++;
                    if(CurrentWGNode != -1)
                    {
                    	FromWGNode = CurrentWGNode;
                        CurrentWGNode = -1;
                    }
                    flag7 = false;
                } else
                if(differentAff)
                {
                	for(mapIterator= map_list.listIterator(); mapIterator.hasNext();)
                    {
                    	mapNode = (NodeMap)mapIterator.next();
                        if(CurrentWGNode == -1 ? mapNode.id == FromWGNode : mapNode.id == CurrentWGNode)
	                        break;
                    }
                    edge_to_direction[currentDir]++;
                    int k3= currentDir;
                    number_of_steps= edge_to_direction[currentDir];
                    for(int j2 = 0; j2 < mapNode.pointerWGtoWG.size(); j2++)
                    {
                    	Edge edge = (Edge)mapNode.pointerWGtoWG.get(j2);
                        if(edge.direction == k3 && (double)edge.steps == number_of_steps)
                        {
                            if (edge.trace == false)
                                edge.trace= true;
                            else if (Rat.training)
                                Rat.nextTraining= true;
                            CurrentWGNode = edge.points_to_node;
                            FromWGNode = -1;
                            for(int j1 = 0; j1 < 8; j1++)
                            	edge_to_direction[j1] = 0.0D;
                            number_of_steps = 0.0D;
                            patNode.mapNodeID = edge.points_to_node;
                            flag7 = false;
                            j2 = mapNode.pointerWGtoWG.size();
                        }
                    }
                    if(edge_to_direction[currentDir] != 0.0D)
                    	edge_to_direction[currentDir]--;
                }
            if(flag7)
            {
              	for(mapIterator= map_list.listIterator(); mapIterator.hasNext();)
                {
                	mapNode = (NodeMap)mapIterator.next();
                    if(CurrentWGNode == -1 ? mapNode.id == FromWGNode : mapNode.id == CurrentWGNode)
	                	break;
                }
                int l2 = check_boundaries_wg(mapNode.wg_x, mapNode.wg_y, direction2angle(currentDir));
                direction.value = angle2direction(l2);
                int i3 = mapNode.wg_x + 50 * direction.x();
                int j3 = mapNode.wg_y + 50 * direction.y();
                boolean flag8 = false;
                boolean flag10 = true;
                while(flag10) 
                {
                	boolean flag9 = true;
                    for(mapIterator= map_list.listIterator(); mapIterator.hasNext();)
                    {
                    	NodeMap mapNodeAux = (NodeMap)mapIterator.next();
                        if(mapNodeAux.wg_x == i3 && mapNodeAux.wg_y == j3)
                        {
                        	if(Math.random() > 0.5D)
                            {
                            	if(l2 == 360)
                                	l2 = 45;
                                else
                                	l2 += 45;
                            } else
                            if(l2 == 45)
                            	l2 = 360;
                            else
                            	l2 -= 45;
                            l2 = check_boundaries_wg(i3, j3, l2);
                            direction.value = angle2direction(l2);
                            i3 += 50 * direction.x(); 
                            j3 += 50 * direction.y(); 
                            flag9 = false;
                            break;
                        }
                    }
                    if(flag9)
	                    flag10 = false;
                }
                node_id_WG++;
                if(!newTrial)
                {
                	mapIterator= map_list.listIterator();
                    if(CurrentWGNode == -1)
                    	while(mapIterator.hasNext()) 
                        {
                        	mapNode = (NodeMap)mapIterator.next();
                            if(mapNode.id == FromWGNode)
                            	break;
                        }
                    else
                        while(mapIterator.hasNext()) 
                        {
                        	mapNode = (NodeMap)mapIterator.next();
                            if(mapNode.id == CurrentWGNode)
                            	break;
                        }
                    edge_to_direction[currentDir]++;
                    number_of_steps= edge_to_direction[currentDir];
                    mapNode.add_edge(i3, j3, currentDir, node_id_WG, (int)number_of_steps, true);
                }
                for(int k1 = 0; k1 < 8; k1++)
                	edge_to_direction[k1] = 0.0D;
                number_of_steps = 0.0D;
                mapNode = new NodeMap(node_id_WG);
                mapIterator.add(mapNode);
                mapNode.wg_x = i3;
                mapNode.wg_y = j3;
                patNode.mapNodeID = mapNode.id;
                CurrentWGNode = mapNode.id;
                FromWGNode = -1;
                newMapNode = true;
            }
        } else
        {
            for(patIterator= pattern_list.listIterator(); patIterator.hasNext();)
            {
            	patNode = (NodePattern)patIterator.next();
                if(patNode.id == CurrentNode)
                	break;
            }
            if(!newTrial && previous_direction == currentDir)
            {
            	for(mapIterator = map_list.listIterator(); mapIterator.hasNext();)
                {
                	mapNode = (NodeMap)mapIterator.next();
                    if(mapNode.id == patNode.mapNodeID)
                    	break;
                }
                winner_x = mapNode.wg_x;
                winner_y = mapNode.wg_y;
                if(CurrentWGNode == -1)
                	CurrentWGNode = FromWGNode;
                if(CurrentWGNode != patNode.mapNodeID)
                {
                	boolean flag4 = true;
                    for(mapIterator = map_list.listIterator(); mapIterator.hasNext();)
                    {
                    	NodeMap mapNodeAux = (NodeMap)mapIterator.next();
                        if(mapNodeAux.id == CurrentWGNode)
                        {
                        	for(int k2 = 0; k2 < mapNodeAux.pointerWGtoWG.size(); k2++)
                            {
                            	Edge edge1 = (Edge)mapNodeAux.pointerWGtoWG.get(k2);
                                if(edge1.points_to_node == patNode.mapNodeID)
                                {
                                    if (edge1.trace == false)
                                        edge1.trace= true;
                                    else if (Rat.training)
                                        Rat.nextTraining= true;
                                    flag4 = false;
                                    k2 = mapNodeAux.pointerWGtoWG.size();
                                }
                            }
                            if(flag4)
                            {
                            	edge_to_direction[currentDir]++;
                                number_of_steps= edge_to_direction[currentDir];
                                mapNodeAux.add_edge(winner_x, winner_y, currentDir, patNode.mapNodeID, (int)number_of_steps, true);
                                number_of_steps = 0.0D;
                            }
                            for(int l1 = 0; l1 < 8; l1++)
                            	edge_to_direction[l1] = 0.0D;
                            break;
                        }
                    }
                }
            }
            CurrentWGNode = patNode.mapNodeID;
            FromWGNode = -1;
        }
        return newMapNode;
    }
    
    public int check_boundaries_wg(int i, int j, int k)
    {
        if(i <= 80)
        {
            if(j <= 80)
                k = 315;
            else
            if(j >= 550)
                k = 45;
            else
                k = 360;
        } else
        if(i >= 550)
        {
            if(j <= 80)
                k = 225;
            else
            if(j >= 550)
                k = 135;
            else
                k = 180;
        } else
        if(j <= 80)
            k = 270;
        else
        if(j >= 550)
            k = 90;
        return k;
    }

    public int direction2angle(int i)
    {
        switch(i)
        {
        case 0:
            return 360;
        case 1:
            return 45;
        case 2:
            return 90;
        case 3:
            return 135;
        case 4:
            return 180;
        case 5:
            return 225;
        case 6:
            return 270;
        case 7:
            return 315;
        }
        return -1;
    }
    
    public int angle2direction(int i)
    {
        switch(i)
        {
        case 360: 
            return 0;
        case 45:
            return 1;
        case 90:
            return 2;
        case 135: 
            return 3;
        case 180: 
            return 4;
        case 225: 
            return 5;
        case 270: 
            return 6;
        case 315: 
            return 7;
        }
        return -1;
    }
    
    
    public boolean readActivationMap(int i, double ad[], boolean aflag[], int j, int k, double d)
    {
    	ListIterator mapIterator;
    	NodeMap mapNode= null;
        double d1 = 0.0D;
        double d2 = 0.0D;
        int j1 = -1;
        int k1 = -1;
        
        if(k == -999)
            if(CurrentWGNode == -1)
                k = FromWGNode;
            else
                k = CurrentWGNode;
        if(k != -1 && i < CYCLING)
        {
            i++;
           	for(mapIterator= map_list.listIterator(); mapIterator.hasNext();)
        	{
        		mapNode = (NodeMap)mapIterator.next();
                if(mapNode.id == k)
                    break;
            }
            k = -1;
            for(int i1 = 0; i1 < mapNode.pointerWGtoWG.size(); i1++)
            {
                Edge edge = (Edge)mapNode.pointerWGtoWG.get(i1);
                for(int l = 0; l < 8; l++)
                    if(edge.direction == l)
                    {
                        d2 = mapNode.w[j][l];
                        if(d2 > d1)
                        {
                            d1 = d2;
                            j1 = l;
                            k1 = edge.points_to_node;
                        }
                    }
                k = k1;
                if(j1 != -1)
                    if(d > mapNode.w[j][j1])
                    {
                        k = -1;
                    } else
                    {
                        d = d2;
                        ad[j1] += d2;
                    }
            }
            if (i==1 && j1 != -1)
                aflag[j1]=true;
            readActivationMap(i, ad, aflag, j, k, d);
        } else
        {
            return false;
        }
        return true;
    }
}