import javax.media.j3d.*;
import javax.vecmath.*;
import com.sun.j3d.utils.geometry.*;
import com.sun.j3d.utils.universe.*;

public class worldBranchGroup extends BranchGroup{
    
    private Vector3f food; 
    private Vector3f topView;
    private Vector3f robotView;
    private Vector3f robotCameraView;
    private Vector3f thirdView;
    private TransformGroup robotBodyTranslationGroup;
    private TransformGroup robotBodyRotationGroup;
    private TransformGroup robotCameraTranslationGroup;
    private TransformGroup robotCameraRotationGroup;
    private TransformGroup foodTranslationGroup;
    private CollisionReporter reporter = new CollisionReporter();
    
    public worldBranchGroup() {
        super();
        this.setCapability(BranchGroup.ALLOW_CHILDREN_EXTEND);
        this.setCapability(BranchGroup.ALLOW_CHILDREN_WRITE);
    }
    
    public void addSphere(float r, Color3f color, float xp, float yp, float zp, boolean collidable) {
        Appearance app = new Appearance();
        Material mat = createMaterial(color);
        
        app.setMaterial(mat);
	Primitive vol = new Sphere(r, app);

	addVolume(vol, xp, yp, zp);
        if(collidable)
            addCollisionBehavior(vol, xp, yp, zp);
    }
    
    public void addCylinder(float r, float h, Color3f color, float xp, float yp, float zp, boolean collidable) {
        Appearance app = new Appearance();
        Material mat = createMaterial(color);
        
        app.setMaterial(mat);
	Primitive vol = new Cylinder(r, h, app);

	addVolume(vol, xp, yp, zp);
        if(collidable)
            addCollisionBehavior(vol, xp, yp, zp);
    }
    
    public void addBox(float x, float y, float z, Color3f color, float xp, float yp, float zp, boolean collidable) {
        Appearance app = new Appearance();
        Material mat = createMaterial(color);
        
        app.setMaterial(mat);
	Primitive vol = new Box(x, y, z, app);

	addVolume(vol, xp, yp, zp);
        if(collidable)
            addCollisionBehavior(vol, xp, yp, zp);
    }
    
    public void addVolume(Primitive vol, float x, float y, float z) {
        Transform3D translate = new Transform3D();
        translate.setTranslation(new Vector3f(x,y,z));
	TransformGroup tg = new TransformGroup(translate);
	tg.addChild(vol);
    
        this.addChild(tg);
        
        tg.setCapability(TransformGroup.ALLOW_TRANSFORM_WRITE);
	tg.setCapability(TransformGroup.ALLOW_TRANSFORM_READ);
    }
    
    int id = 0;
    public void addCollisionBehavior(Primitive vol, double x, double y, double z) {
        Behavior collisionbehavior = new CollisionBehavior(reporter, vol, id);
        collisionbehavior.setSchedulingBounds(new BoundingSphere(new Point3d(x, y, z), 2d));
        this.addChild(collisionbehavior);
        id++;
    }
    
    protected Material createMaterial(Color3f color) {
	Material mat = new Material();
	mat.setDiffuseColor(color);
	mat.setSpecularColor(color);
	mat.setShininess(0f);
        
        return mat;
    }
    
    public void setTopView(Vector3f vector) {
        topView = vector;
    }
    
    public void setThirdView(Vector3f vector) {
        thirdView = vector;
    }
 
    
    
    
    public void addFood(float r, float h, Color3f color, float xp, float yp, float zp, boolean collidable) {
        
        food = new Vector3f(xp,yp,zp);    
        Appearance app = new Appearance();
        Material mat = createMaterial(color);
        
        app.setMaterial(mat);
	Primitive vol = new Cylinder(r, h, app);

	
        Transform3D translate = new Transform3D();
        translate.setTranslation(new Vector3f(xp,yp,zp));
	foodTranslationGroup = new TransformGroup(translate);
	foodTranslationGroup.addChild(vol);
    
        this.addChild(foodTranslationGroup);
        
        foodTranslationGroup.setCapability(TransformGroup.ALLOW_TRANSFORM_WRITE);
	foodTranslationGroup.setCapability(TransformGroup.ALLOW_TRANSFORM_READ);

    }

    public void setRobotView(Vector3f vector) {
        Primitive cylinder;
        
        robotView = vector;
        robotCameraView = new Vector3f(vector.x, vector.y+0.05f, vector.z);
        BranchGroup branch = new BranchGroup();
        
        Appearance app = new Appearance();
        Material mat = createMaterial(new Color3f(0.5f,0f,1f));
        app.setMaterial(mat);
        
	cylinder = new Cylinder(0.025f, 0.1f, app);
        
        Transform3D rotate = new Transform3D();
        rotate.rotX(Math.PI/180.0d*90);
        this.robotBodyRotationGroup = new TransformGroup();
        this.robotBodyRotationGroup.setCapability(TransformGroup.ALLOW_TRANSFORM_READ);
        this.robotBodyRotationGroup.setCapability(TransformGroup.ALLOW_TRANSFORM_WRITE);
        this.robotBodyRotationGroup.setTransform(rotate);
        this.robotBodyRotationGroup.addChild(cylinder);
        
        Transform3D translate = new Transform3D();
        translate.setTranslation(robotView);
        this.robotBodyTranslationGroup = new TransformGroup();
        this.robotBodyTranslationGroup.setCapability(TransformGroup.ALLOW_TRANSFORM_READ);
        this.robotBodyTranslationGroup.setCapability(TransformGroup.ALLOW_TRANSFORM_WRITE);
        this.robotBodyTranslationGroup.setTransform(translate);
        this.robotBodyTranslationGroup.addChild(this.robotBodyRotationGroup);
            
        this.addChild(this.robotBodyTranslationGroup);
        
        app = new Appearance();
        mat = createMaterial(new Color3f(0.5f,0.5f,0f));
        app.setMaterial(mat);
        
	cylinder = new Cone(0.025f, 0.1f, app);
        
        rotate = new Transform3D();
        rotate.rotX(Math.PI/180.0d*90);
        this.robotCameraRotationGroup = new TransformGroup();
        this.robotCameraRotationGroup.setCapability(TransformGroup.ALLOW_TRANSFORM_READ);
        this.robotCameraRotationGroup.setCapability(TransformGroup.ALLOW_TRANSFORM_WRITE);
        this.robotCameraRotationGroup.setTransform(rotate);
        this.robotCameraRotationGroup.addChild(cylinder);
        
        translate = new Transform3D();
        translate.setTranslation(robotCameraView);
        this.robotCameraTranslationGroup = new TransformGroup();
        this.robotCameraTranslationGroup.setCapability(TransformGroup.ALLOW_TRANSFORM_READ);
        this.robotCameraTranslationGroup.setCapability(TransformGroup.ALLOW_TRANSFORM_WRITE);
        this.robotCameraTranslationGroup.setTransform(translate);
        this.robotCameraTranslationGroup.addChild(this.robotCameraRotationGroup);
        this.addChild(this.robotCameraTranslationGroup);
        
    }
    
    public void moveRobot(Vector3f vector) {      
        robotView.x += vector.x;
        robotView.y += vector.y;
        robotView.z += vector.z;
        
        robotCameraView.x += vector.x;
        robotCameraView.y += vector.y;
        robotCameraView.z += vector.z;
        
        Transform3D translate = new Transform3D();
        translate.setTranslation(robotView);
        
        this.robotBodyTranslationGroup.setTransform(translate);
        
        translate = new Transform3D();
        translate.setTranslation(robotCameraView);
        
        this.robotCameraTranslationGroup.setTransform(translate);
    }
   public void startRobot(Vector3f vector) {      
        robotView= vector;
        robotCameraView = new Vector3f(vector.x, vector.y+0.05f, vector.z);       
        Transform3D translate = new Transform3D();
        translate.setTranslation(robotView);       
        this.robotBodyTranslationGroup.setTransform(translate);        
        translate = new Transform3D();
        translate.setTranslation(robotCameraView);        
        this.robotCameraTranslationGroup.setTransform(translate);
    }
    public void moveFood(Vector3f vector) {      
        food.x += vector.x;
        food.y += vector.y;
        food.z += vector.z;
        
        Transform3D translate = new Transform3D();
        translate.setTranslation(food);
     

        this.foodTranslationGroup.setTransform(translate);
    }

    public void rotateRobotBody(double degrees) {
        
        Transform3D rotate = new Transform3D();
        rotate.rotZ(Math.toRadians(-degrees));
        
        Transform3D temp = new Transform3D();
        this.robotBodyRotationGroup.getTransform(temp);
        
        temp.mul(rotate);
        this.robotBodyRotationGroup.setTransform(temp);   
        rotate = new Transform3D();
        rotate.rotZ(Math.toRadians(-degrees));
        
        temp = new Transform3D();
        this.robotCameraRotationGroup.getTransform(temp);
        
        temp.mul(rotate);
        this.robotCameraRotationGroup.setTransform(temp);   
    }
    
    public void rotateRobotCamera(double degrees) {
        
        Transform3D rotate = new Transform3D();
        rotate.rotZ(Math.toRadians(-degrees));
        
        Transform3D temp = new Transform3D();
        this.robotCameraRotationGroup.getTransform(temp);
        
        temp.mul(rotate);
        this.robotCameraRotationGroup.setTransform(temp);   
    }
        
    public Vector3f getTopView() {
        return topView;
    }
    
    public Vector3f getRobotView() {
        return robotView;
    }
   
    public Vector3f getRobotCameraView() {
        return robotCameraView;
    }
    
    public Vector3f getThirdView() {
        return thirdView;
    }

    public Vector3f getFood() {
        return food;
    }

    public boolean isCollided() {
        return reporter.isCollided();
    }
    
}
